package org.wffm.gassan.model;

/*
 * GDirectorImpl.java - Short Description
 * Copyright (C) 2000 WFFM
 * koutei@erl.ee.musashi-tech.ac.jp
 * http://www.wffm.org/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Properties;
import java.util.Calendar;
import java.util.ArrayList;

import jp.gr.java_conf.wffm.gassan.data.GADFileSystem;
import org.wffm.gassan.model.translator.GTranslator;
import org.wffm.gassan.model.translator.GSMLTranslator;
import org.wffm.gassan.model.translator.GPDTranslator;
import org.wffm.gassan.model.scenario.GElement;
import org.wffm.gassan.model.scenario.GElementList;

/**
  * Rpo
  * @version 1.0.1, 2000/12/26
  * @author Yosuke Sato
  */
public class GDirectorImpl implements GDirector{
	//
	//  ֘A
	//
	//=========================================================================
	private GStage _stage = null;
	private GElement _env = null;
	
	//
	//  W
	//
	//=========================================================================
	private GTranslator _translator = null;
	
	//
	//  [Jϐ
	//
	//=========================================================================
	private File bookmarkFile = null;

	public static final int maximumOfBookmark = 5;
	public static final String softDir = ".wffm/gassan/3_1";
	public static final String saveFileName = "save.ser";
/**
  * RXgN^
  */
public GDirectorImpl() {
}
private File createBookmarkFile(){
	//O: envnullłȂ
	//: ubN}[Nt@C쐬
	
	File ret = null;

	if(_env != null){
		//ViI擾	
		GElementList list = _env.getChildElements();
		list.first();
		list.search("scenario");
		GElement element = list.item();
		String scenarioName = element.getAttributeValue("name");
		
		//user.home擾
		Properties env = System.getProperties();
		if(scenarioName == null)	scenarioName = "default";
		String key = "user.home";
		String userHomeDir = env.getProperty(key);
		
		//[UfBNg쐬
		String dirName = userHomeDir +"/"+ softDir +"/"+ scenarioName;
		File dir = new File(dirName);
		dir.mkdirs();
	
		//ubN}[Nt@C쐬
		ret =  new File(dirName +"/"+ saveFileName);
	}
	else{
		System.err.println("Director::createBookmarkFile Oᔽ");
		ret = null;
	}
		
	return ret;
}
private GTranslator createGADTranslator(File gadFile,String scenarioName){
	GTranslator ret = null;
	
	if(gadFile.exists()){
		//e|fBNg쐬
		Properties env = System.getProperties();
		String key = "user.home";
		String userHomeDir = env.getProperty(key);
		String tempdir = userHomeDir +"/"+ softDir +"/"+ scenarioName +"/temp";
		File gpdFile =	GADFileSystem.readFile(gadFile,tempdir);
		URL gpdURL = null;
		try {
			gpdURL = gpdFile.toURL();
		} catch ( MalformedURLException ex ) {
			gpdURL = null;
		}
		if(gpdURL != null){
			GPDTranslator gpdTranslator = new GPDTranslator(gpdURL);
			ret = (GTranslator)gpdTranslator;
		}
	}

	return ret;
}
private GTranslator createTranslator(){
	//O: envnullłȂ
	//: K؂Ȗ|҂쐬
	
	GTranslator ret = null;

	if(_env != null){
		//ViIt@C擾
		GElementList list = _env.getChildElements();
		list.first();
		list.search("scenario");
		GElement element = list.item();
		String scenarioName = element.getAttributeValue("name");
		String scenarioFile = element.getAttributeValue("gsmlFile");
		
		URL scenarioURL = null;
		String urlString = _env.getBaseURL().toString() + scenarioFile;
		try{
			scenarioURL = new URL(urlString);
		}catch(MalformedURLException e){
			scenarioURL = null;
		}
			
		//t@C^Cv
		int separatorIndex = scenarioFile.lastIndexOf( "." );
		String fileType = scenarioFile.substring( separatorIndex + 1, scenarioFile.length() );

		if(fileType.equals("gsml")){
			ret  = new GSMLTranslator(scenarioURL);
		}
		else if(fileType.equals("gpd")){
			ret = new GPDTranslator(scenarioURL);
		}
		else if(fileType.equals("gad")){
			ret = createGADTranslator(new File(scenarioURL.getFile()),scenarioName);
		}
	}
	else{
		System.err.println("Director::createBookmarkFile OᔽD");
		ret = null;
	}
	
	return ret;
}
/**
  * Q擾
  * @return	Q
  */
public Bookmark[] getBookmarks(){
	Bookmark[] ret = null;
	
	//𐶐
	try{
		ret = loadBookmarks(bookmarkFile);
	}
	catch(IOException e){
		System.err.println("can not load savefile.");
		ret = new Bookmark[maximumOfBookmark];
	}
	catch(Exception e){
		System.err.println("can not load savefile.");
		ret = new Bookmark[maximumOfBookmark];
	}
	return ret;
}
/**
  * 
  * @param	_env	
  */
public void init(GElement _env) {
	this._env = _env;

	bookmarkFile = createBookmarkFile();
	_translator = createTranslator();
}
private Bookmark[] loadBookmarks(File file) throws IOException,Exception{
	Bookmark[] ret = null;
	
	try{
		ArrayList list = new ArrayList();
		FileInputStream fis = new FileInputStream(file);
		ObjectInputStream ois = new ObjectInputStream(fis);
		while( fis.available() != 0){
			Bookmark bookmark = (Bookmark)ois.readObject();
			list.add(bookmark);
		}
		ois.close();
		fis.close();
		ret = (Bookmark[])list.toArray(new Bookmark[list.size()]);
	}
	catch(IOException e){throw e;}
	catch(Exception e){throw e;}

	return ret;
}
/**
  * V[Iʒm
  */
public void notifySceneEnd(){
	//O: translator,stagenullłȂ
	//: stage֎̃V[w
	
	if(_translator != null && _stage != null){
		_translator.next();
		_stage.instruct(_translator.scene());
	}
	else{
		System.err.println("Director :: notifySceneEnd Oᔽ");
	}
}
/**
  * f[^
  * @param	operation	f[^
  */
public void operation(String operation){
	if(_translator != null){
		_translator.operation(operation);
	}
}
private void saveBookmarks(File file,Bookmark[] bookmarks) throws IOException,Exception{
	try{
		ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(file));
		for(int i = 0 ; i < bookmarks.length ; i++){
			oos.writeObject(bookmarks[i]);
		}
		oos.close();
	}
	catch(IOException e){throw e;}
	catch(Exception e){throw e;}
}
/**
  * ㉉ۑ
  * @param	bookmarkNumber	ԍ
  */
public void savePerform(int bookmarkNumber){
	if( 0 <= bookmarkNumber && bookmarkNumber <maximumOfBookmark){
		if(_translator != null && _translator.scene() != null){
			GElement scene = _translator.scene();
			String sceneName = scene.getAttributeValue("name");
			
			Bookmark[] bookmarks = getBookmarks();
			Bookmark bookmark = new Bookmark(sceneName,Calendar.getInstance(),_translator);
			bookmarks[bookmarkNumber] = bookmark;
			try{
				saveBookmarks(bookmarkFile,bookmarks);
			} catch(IOException e){
				System.err.println("can not save.");
			} catch(Exception e){
				System.err.println("can not save.");
			}
		}
	}
	else{
		System.err.println("bookmarkNumber͈͊OłD");
	}
}
/**
  * ݒ
  * @param	stage	
  */
public void setStage(GStage _stage){
	this._stage = _stage;
}
/**
  * ㉉Jn
  */
public void startPerform(){
	if(_stage != null ){
		_translator = createTranslator();
		if(_translator != null){
			_translator.first();
			_stage.instruct(_translator.scene());
		}
		else{
			_stage.instruct(null);
		}
	}
}
/**
  * ㉉Jn(肩)
  * @param	bookmarkNumber	ԍ
  */
public void startPerform(int bookmarkNumber){

	if( 0 <= bookmarkNumber && bookmarkNumber <maximumOfBookmark){
		Bookmark[] bookmarks = getBookmarks();
		Bookmark bookmark = bookmarks[bookmarkNumber];
		
		if(bookmark != null){
			_translator = bookmark.getGTranslator();
			if(_stage != null){
				_stage.instruct(_translator.scene());
			}
		}
	}
	else{
		System.err.println("bookmarkNumber͈͊OłD");
	}
}
}
