/* Public Domain Sources */

package org.wffm.gassan.view.loader.bmp;

import java.awt.image.ImageConsumer;
import java.awt.image.ColorModel;
import java.awt.image.IndexColorModel;


/**
  * BMPIndexColor
  * @version 1.0.0, 2000/12/01
  * @author D.Miki
  */
class BMPIndexColor extends Bitmap {

  static Bitmap load(BitmapFileHeader bmpfh,BitmapInfoHeader bmpih,WordableDataInputStream is) throws Exception {
	byte[] red   = new byte[bmpih.getColorCount()];
	byte[] green = new byte[bmpih.getColorCount()];
	byte[] blue  = new byte[bmpih.getColorCount()];
	for(int i=0;i<bmpih.getColorCount();++i){
	  blue[i]  = is.readByte();
	  green[i] = is.readByte();
	  red[i]   = is.readByte();
	  is.skipBytes(1);
	}
	byte[] vram=null;
	switch(bmpih.getColorCount()){
	case BitmapInfoHeader.MONO_COLOR:
	  vram=BMPIndexColor.loadBytesMono(bmpih.getWidth(),bmpih.getHeight(),is);
	  break;
	case BitmapInfoHeader.VGA_COLOR:
	  vram=(bmpih.isRLE4Compression())?
		(BMPIndexColor.loadBytes16RLE(bmpfh.getFileSize()-bmpfh.getOffsetByte(),bmpih.getWidth(),bmpih.getHeight(),is)):
		(BMPIndexColor.loadBytes16(bmpih.getWidth(),bmpih.getHeight(),is))
	  ;
	  break;
	case BitmapInfoHeader.FIX_COLOR:
	  vram=(bmpih.isRLE8Compression())?
		(BMPIndexColor.loadBytes256RLE(bmpfh.getFileSize()-bmpfh.getOffsetByte(),bmpih.getWidth(),bmpih.getHeight(),is)):
		(BMPIndexColor.loadBytes256(bmpih.getWidth(),bmpih.getHeight(),is))
	  ;
	  break;
	default:
	  throw new BMPException("BMPIndexColor#load: Index"+bmpih.getColorCount()+"Color is not supported.");
	}
	if(vram==null)throw new BMPException("BMPIndexColor#load: Failed getBytes.");
	return new BMPIndexColor(bmpfh,bmpih,vram,red,green,blue);
  }          

  private static byte[] loadBytesMono(int w,int h,WordableDataInputStream is) throws Exception {
	int skip=(((w+7)/8)%4!=0)?4-((w+7)/8)%4:0;
	byte[] buf=new byte[w*h];
	for(int j=0;j<h;++j){
	  int ofs=(h-1-j)*w;
	  int d=0;
	  for(int i=0;i<w/8;++i){
		byte b=is.readByte();
		int mask=0x80;
		for(int k=0;k<8;++k){
		  buf[ofs+d++]=(byte)((b&mask)>>(7-k));
		  mask>>=1;
		}
	  }
	  if(w%8!=0){
		byte b=is.readByte();
		int mask=0x80;
		for(int k=0;k<w%8;++k){
		  buf[ofs+d++]=(byte)((b&mask)>>(7-k));
		  mask>>=1;
		}
	  }
	  is.skipBytes(skip);
	}
	return buf;
  }          
  private static byte[] loadBytes16(int w,int h,WordableDataInputStream is) throws Exception {
	int skip=(((w+1)/2)%4!=0)?4-((w+1)/2)%4:0;
	byte[] buf=new byte[w*h];
	for(int j=0;j<h;++j){
	  int ofs=(h-1-j)*w;
	  int d = 0;
	  for(int i=0;i<w/2;i++){
		int b=is.readByte();
		buf[ofs+d++]=(byte)((b&0xf0)>>4);
		buf[ofs+d++]=(byte)(b&0x0f);
	  }
	  if(w%2!=0)buf[ofs+d]=(byte)((is.readByte()&0xf0)>>4);
	  is.skipBytes(skip);
	}
	return buf;
  }          
  private static byte[] loadBytes256(int w,int h,WordableDataInputStream is) throws Exception {
	int skip=(w%4!=0)?4-w%4:0;
	byte[] buf=new byte[w*h];
	for(int j=0;j<h;++j){
	  int ofs=(h-1-j)*w;
	  for(int i=0;i<w;++i)buf[ofs+i]=is.readByte();
	  is.skipBytes(skip);
	}
	return buf;
  }          

  private static byte[] loadBytes16RLE(int size,int w,int h,WordableDataInputStream is) throws Exception {
	byte buf[]=new byte[w*h];
	int i=0;
	int d=0;
	int ofs=(h-1)*w;
	--h;
	while(i<size){
	  int b1 = is.readUnsignedByte();
	  int b2 = is.readUnsignedByte();
	  i+=2;
	  if(b1>0){
		for(int j=0;j<b1;++j)buf[ofs+d++]=(j%2==0)?(byte)((b2&0xf0)>>4):(byte)(b2&0x0f);
	  }else{
		if(b2==0){
		  ofs=(h-1)*w;
		  --h;
		  d=0;
		  if(h<0)break;
		}else if(b2==1){
		  break;
		}else if(b2==2){
		  int b3=is.readUnsignedByte();
		  int b4=is.readUnsignedByte();
		  i+=2;
		  if(b3>0)d+=b3-1;
		  if(b4>0){
			ofs-=w*b4;
			h-=b4;
		  }
		}else{
		  int b3=0;
		  for(int j=0;j<b2;++j){
			if(j%2==0){
			  b3=is.readByte();
			  ++i;
			}
			buf[ofs+d++]=(j%2==0)?(byte)((b3&0xf0)>>4):(byte)(b3&0x0f);
		  }
		  if(((b2+1)/2)%2!=0){
			is.skipBytes(1);
			++i;
		  }
		}
	  }
	}
	return buf;
  }          
  private static byte[] loadBytes256RLE(int size,int w,int h,WordableDataInputStream is) throws Exception {
	byte buf[]=new byte[w*h];
	int i = 0;
	int d = 0;
	int ofs=(h-1)*w;
	--h;
	while(i<size){
	  int b1=is.readUnsignedByte();
	  int b2=is.readUnsignedByte();
	  i+=2;
	  if(b1>0){
		for(int j=0;j<b1;++j)buf[ofs+d++]=(byte)b2;
	  }else{
		if(b2==0){
		  ofs=(h-1)*w;
		  --h;
		  d=0;
		  if(h<0)break;
		}else if(b2==1){
		  break;
		}else if(b2==2){
		  int b3=is.readUnsignedByte();
		  int b4=is.readUnsignedByte();
		  i+=2;
		  if(b3>0)d+=b3-1;
		  if(b4>0){
			ofs-=w*b4;
			h-=b4;
		  }
		}else{
		  for(int j=0;j<b2;++j){
			buf[ofs+d++]=is.readByte();
			++i;
		  }
		  if(b2%2!=0){
			is.skipBytes(1);
			++i;
		  }
		}
	  }
	}
	return buf;
  }          

  private byte[] vram;

  private BMPIndexColor(BitmapFileHeader bmpfh,BitmapInfoHeader bmpih,byte[] vram,byte[] red,byte[] green,byte[] blue){
	super(bmpfh,bmpih,new IndexColorModel(8,bmpih.getColorCount(),red,green,blue));
	this.vram=vram;
  }          

  protected void setPixelData() {
	BitmapInfoHeader bih=this.getInfoHeader();
	this.getImageConsumer().setPixels(
	  0,0,bih.getWidth(),bih.getHeight(),this.getColorModel(),this.vram,0,bih.getWidth()
	);
  }          

}