/* Public Domain Sources */

package org.wffm.gassan.view.loader.bmp;

import java.awt.image.ImageProducer;
import java.awt.image.ImageObserver;
import java.awt.image.ImageConsumer;
import java.awt.image.PixelGrabber;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.DirectColorModel;
import java.awt.image.IndexColorModel;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.BufferedInputStream;
import java.util.Hashtable;
import javax.swing.ImageIcon;



/**
  * Bitmap
  * @version 1.0.0, 2000/12/01
  * @author D.Miki
  */
public abstract class Bitmap extends Object implements ImageProducer {

  public static Bitmap load(InputStream src) throws Exception {
	WordableDataInputStream is = new WordableDataInputStream(new BufferedInputStream(src));
	BitmapFileHeader bmpfh=BitmapFileHeader.load(is);
	BitmapInfoHeader bmpih=BitmapInfoHeader.load(is);
	if(bmpih.isFullColor()){
	  return BMPFullColor.load(bmpfh,bmpih,is);
	}else{
	  return BMPIndexColor.load(bmpfh,bmpih,is);
	}
  }          

  public static Bitmap load(String filename) throws Exception {
	InputStream is=new FileInputStream(filename);
	try{
	  return Bitmap.load(is);
	}catch(Exception e){
	  throw e;
	}finally{
	  try{is.close();}catch(Exception ex){}
	}
  }          

  public static BufferedImage loadImage(InputStream src) throws Exception {
	return bitmap2image(Bitmap.load(src));
  }          
  public static BufferedImage loadImage(String filename) throws Exception {
	return bitmap2image(Bitmap.load(filename));
  }          
  private static BufferedImage bitmap2image(Bitmap bmp) throws Exception {
	int w=bmp.getWidth();
	int h=bmp.getHeight();
	int[] pixs=new int[w*h];
	PixelGrabber pxgb=new PixelGrabber(bmp,0,0,w,h,pixs,0,w);
	pxgb.grabPixels();
	if((pxgb.getStatus()&ImageObserver.ABORT)!=0)throw new BMPException("Bitmap#loadImage: Failed pixel grabbing.");
	BufferedImage img=new BufferedImage(w,h,BufferedImage.TYPE_INT_RGB);
	img.setRGB(0,0,w,h,pixs,0,w);
	return img;
  }          
  public static ImageIcon loadIcon(String filename) throws Exception {
	return new ImageIcon(Bitmap.loadImage(filename));
  }          
  public static ImageIcon loadIcon(InputStream src) throws Exception {
	return new ImageIcon(Bitmap.loadImage(src));
  }          

  private BitmapFileHeader bmpfh;
  private BitmapInfoHeader bmpih;
  private ColorModel colormodel;
  private ImageConsumer ic;

  protected Bitmap(BitmapFileHeader bmpfh,BitmapInfoHeader bmpih,ColorModel colormodel){
	super();
	this.bmpfh=bmpfh;
	this.bmpih=bmpih;
	this.colormodel=colormodel;
	this.ic=null;
  }          

  protected abstract void setPixelData();          

  protected BitmapFileHeader getFileHeader(){return this.bmpfh;}          
  protected BitmapInfoHeader getInfoHeader(){return this.bmpih;}          
  protected ImageConsumer getImageConsumer(){return this.ic;}          
  protected ColorModel getColorModel(){return this.colormodel;}          

  public int getWidth(){return this.getInfoHeader().getWidth();}          
  public int getHeight(){return this.getInfoHeader().getHeight();}          

  public synchronized void addConsumer(ImageConsumer ic){
	this.ic = ic;
	if (this.ic!=null){
	  this.ic.setDimensions(this.bmpih.getWidth(),this.bmpih.getHeight());
	  this.ic.setProperties(new Hashtable());
	  this.ic.setColorModel(this.colormodel);
	  this.ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT |ImageConsumer.COMPLETESCANLINES |
					   ImageConsumer.SINGLEPASS |ImageConsumer.SINGLEFRAME);
	  this.setPixelData();
	  this.ic.imageComplete(ImageConsumer.STATICIMAGEDONE);
	}
	this.ic = null;
  }          
  public void startProduction(ImageConsumer ic){addConsumer(ic);}          
  public synchronized boolean isConsumer(ImageConsumer ic){return ic == this.ic;}          
  public synchronized void removeConsumer(ImageConsumer ic){if(this.ic == ic)this.ic = null;}          
  public void requestTopDownLeftRightResend(ImageConsumer ic){}          

}