package org.wffm.gsmlviewer.factory;

/*
 * PaneFactory.java - Short Description
 * Copyright (C) 2000-2001 WFFM
 * koutei@erl.ee.musashi-tech.ac.jp
 * http://www.wffm.org/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.Dimension;
 
import javax.swing.JPanel;

import org.wffm.gsmlviewer.GSMLViewer;
import org.wffm.gsmlviewer.GFrame;
import org.wffm.gsmlviewer.GFrameController;
import org.wffm.gsmlviewer.pane.ImagePane;
import org.wffm.gsmlviewer.pane.TextPane;
import org.wffm.gsmlviewer.pane.TextPaneController;

import org.wffm.gassan.model.GDirector;
import org.wffm.gassan.model.GDirectorImpl;
import org.wffm.gassan.model.scenario.GElement;
import org.wffm.gassan.model.scenario.GElementList;

/**
  * PaneFactoryNX
  * @version 1.1.0, 2001/01/21
  * @author Yosuke Sato
  */
public abstract class ViewFactory {
	//
	//  
	//
	//=========================================================================
	private JPanel panel = null;
	private TextPane textPane = null;
	private ImagePane imagePane = null;
	private GDirector director = null;
	private GElement env = null;
	private GSMLViewer aGSMLViewer = null; 
	private GFrame aGFrame = null;
	private GFrameController aGFrameController = null;
	private TextPaneController aTextPaneController = null;
				
	public static final String packageName = "org.wffm.gsmlviewer.factory.";
	private final static Dimension defaultViewSize = new Dimension(640,480);
/**
  * ViewFactorypNX𐶐
  * @param	className	ViewFactorypNX̖O
  * @return	ꂽViewFactory
  */
public static ViewFactory create(String className) {
	ViewFactory ret = null;
	
	try{
		ret = (ViewFactory)Class.forName(className).newInstance();
	}
	catch(Exception e){
		ret = null;
	}

	//pbP[W⊮@\
	if(ret == null){
		try{
			ret = (ViewFactory)Class.forName(packageName + className).newInstance();
		}
		catch(Exception e){
			System.err.println("Can not create ViewFactory : " + className );
			ret = null;
		}
	}
	
	return ret;
}
/**
  * JPanel擾
  * @return	JPanel
  */
public JPanel getContentPane(){
	if(panel == null){
		panel = new JPanel();
	}
	return panel;
}
public GElement getEnv(){
	return env;
}
public GDirector getGDirector(){
	if(director == null){
		director = new GDirectorImpl();
	}
	return director;
}
public GFrame getGFrame(){
	if(aGFrame == null){
		aGFrame = new GFrame();
	}
	return aGFrame;
}
public GFrameController getGFrameController(){
	if(aGFrameController == null){
		aGFrameController = new GFrameController(getGSMLViewer(),getGFrame());
	}
	return aGFrameController;
}
public GSMLViewer getGSMLViewer(){
	if(aGSMLViewer == null){
		aGSMLViewer = new GSMLViewer();
	}
	return aGSMLViewer;
}
/**
  * ImagePane擾
  * @return	ImagePane
  */
public ImagePane getImagePane(){
	if(imagePane == null){
		imagePane = new ImagePane();
	}
	return imagePane;
}
/**
  * GEMLʃTCY擾
  * @param	env	GEML
  * @return	ʃTCY
  */
public static Dimension getSize(GElement env){
	Dimension ret = null;

	GElementList list = env.getChildElements();
	GElement item = null;
		
	list.first();
	list.search("view");
	item = list.item();

	//TCYݒ

	if(item != null){
		String width = item.getAttributeValue("width");
		String height = item.getAttributeValue("height");
		
		if(width != null && height != null){
			try{
				ret = new Dimension(Integer.parseInt(width),Integer.parseInt(height));
			}catch(NumberFormatException e){
				System.err.println("View Size Error...");
				ret = null;
			}
		}
	}
	if(ret == null){
		ret = defaultViewSize;
	}
	
	return ret;
}
/**
  * TextPane擾
  * @return	TextPane
  */
public TextPane getTextPane(){
	if(textPane == null){
		textPane = new TextPane();
	}
	return textPane;
}
public TextPaneController getTextPaneController(){
	if(aTextPaneController == null){
		aTextPaneController = new TextPaneController(getGSMLViewer(),getTextPane());
	}
	return aTextPaneController;
}
/**
  * 
  * @param	env	
  */
public void init(GElement env){
	this.env = env;
	getGDirector().init(env);
	init_pane();
}
/**
  * pl
  */
protected abstract void init_pane();
}
